/* $Workfile: OtxHdlcD.h $   $Revision: 25 $    $Modtime: 9/07/01 11:17a $  */
#ifndef OTX_HDLC_DEF_H
#define OTX_HDLC_DEF_H
/*******************************************************************************

  @doc PublicHwApi OtxHdlcD

  Copyright (C) 1998 - 2001 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module
  OtxHdlcD | Constants, Macros, and Types for the HDLC Functions

  @contents2
  OtxHdlcD.h |

  @index enum type struct | OtxHdlcD

  @contents2
  OTX Hdlc Device Macros, Constants, and Types |

  @head3 OtxHdlcXXX Macro, Constant, and Type Reference |

  @index enum type struct | OtxHdlcD

*******************************************************************************/

#include "OtxType.h"

#ifdef __cplusplus
  extern "C" {
#endif

#ifdef __linux__
#pragma pack(1)
#else
#pragma pack(push,1)
#endif

#if 0
#define OD_HDLC_MAX_TX_HDLC_FRAME_LEN    8192
#define OD_HDLC_MAX_RX_HDLC_FRAME_LEN    8192
#else // Temporary reduced. Will be brought back to 8192 in future releases
#define OD_HDLC_MAX_TX_HDLC_FRAME_LEN    2048
#define OD_HDLC_MAX_RX_HDLC_FRAME_LEN    2048
#endif

#define OTX_HDLC_MAX_REPEAT_DATA_LEN    10


// Filter masks in the SS7 mode
// @type OTX_HDLC_SS7_FILTER_FISU | Filters identical consecutive FISU frames (LI=0)
#define OTX_HDLC_SS7_FILTER_FISU             0x01
// @type OTX_HDLC_SS7_FILTER_SHORT_LSSU | Filters identical consecutive short LSSU frames (LI=1)
#define OTX_HDLC_SS7_FILTER_SHORT_LSSU       0x02
// @type OTX_HDLC_SS7_FILTER_LONG_LSSU | Filters identical consecutive long LSSU frames (LI=2)
#define OTX_HDLC_SS7_FILTER_LONG_LSSU        0x04
// @type OTX_HDLC_SS7_FILTER_MSU | Filters all MSU frames (LI>2)
#define OTX_HDLC_SS7_FILTER_MSU              0x08
// @type OTX_HDLC_SS7_FILTER_FISU_ALL | Filters all FISU frames (LI=0)
#define OTX_HDLC_SS7_FILTER_FISU_ALL         0x10
// @type OTX_HDLC_SS7_FILTER_SHORT_LSSU_ALL | Filters all short LSSU frames (LI=1)
#define OTX_HDLC_SS7_FILTER_SHORT_LSSU_ALL   0x20
// @type OTX_HDLC_SS7_FILTER_LONG_LSSU_ALL | Filters all long LSSU frames (LI=2)
#define OTX_HDLC_SS7_FILTER_LONG_LSSU_ALL    0x40


#define OTX_HDLC_SS7_CB_FISU 3
#define OTX_HDLC_SS7_CB_SHORT_LSSU 4
#define OTX_HDLC_SS7_CB_LONG_LSSU 5

// FISU Element Extraction Macros
#define OTX_HDLC_SS7_BSN_MASK              0x0000007F  // Backward Sequence Number
#define OTX_HDLC_SS7_BIB_MASK              0x00000080  // Backward Indicator Bit
#define OTX_HDLC_SS7_FSN_MASK              0x00007F00  // Forward Sequence Number
#define OTX_HDLC_SS7_FIB_MASK              0x00008000  // Forward Indicator Bit
#define OTX_HDLC_SS7_LI_MASK               0x003F0000  // Length Indication

#define OTX_HDLC_SS7_FISU_EXTRACT_BSN( bsn, buf32)     bsn = (OTX_UINT8) buf32 & OTX_HDLC_SS7_BSN_MASK

#define OTX_HDLC_SS7_FISU_EXTRACT_BIB( bib, buf32)     bib = (OTX_UINT8) ((buf32 & OTX_HDLC_SS7_BIB_MASK) >> 7)

#define OTX_HDLC_SS7_FISU_EXTRACT_FSN( fsn, buf32)     fsn = (OTX_UINT8) ((buf32 & OTX_HDLC_SS7_FSN_MASK) >> 8)

#define OTX_HDLC_SS7_FISU_EXTRACT_FIB( fib, buf32)     fib = (OTX_UINT8) ((buf32 & OTX_HDLC_SS7_FIB_MASK) >> 15)

#define OTX_HDLC_SS7_FISU_EXTRACT_LI( li, buf32)       li =  (OTX_UINT8) ((buf32 & OTX_HDLC_SS7_LI_MASK) >> 16)


/*---------------------------------------------------------------------------*\
  @enum 
    HDLC Receiver Attributes
\*---------------------------------------------------------------------------*/
enum OtxHdlcReceiverAttributesE {
  OTX_ATTR_HDLC_RECEIVER_FIFO_SIZE_FRAMES           = OTX_ATTR_END +  0,  // @emem (OTX_ATTRIB_UINT32) Size of allocated buffer for the HDLC Receiver Fifo (in number of frames)
  OTX_ATTR_HDLC_RECEIVER_FIFO_SIZE_OCTETS           = OTX_ATTR_END +  1,  // @emem (OTX_ATTRIB_UINT32) Size of allocated buffer for the HDLC Receiver Fifo (in number of bytes)
  OTX_ATTR_HDLC_RECEIVER_FIFO_PEAK_USAGE_OCTETS     = OTX_ATTR_END +  2,  // @emem (OTX_ATTRIB_UINT32) Peak usage (in octets) of the HDLC Receiver Fifo
  OTX_ATTR_HDLC_RECEIVER_FIFO_PEAK_USAGE_FRAMES     = OTX_ATTR_END +  3,  // @emem (OTX_ATTRIB_UINT32) Peak usage (in frames) of the HDLC Receiver Fifo
  OTX_ATTR_HDLC_RECEIVER_FIFO_CURRENT_USAGE_OCTETS  = OTX_ATTR_END +  4,  // @emem (OTX_ATTRIB_UINT32) Current usage (in octets) of the HDLC Receiver Fifo
  OTX_ATTR_HDLC_RECEIVER_FIFO_CURRENT_USAGE_FRAMES  = OTX_ATTR_END +  5,  // @emem (OTX_ATTRIB_UINT32) Current usage (in frames) of the HDLC Receiver Fifo
  OTX_ATTR_HDLC_RECEIVER_FIFO_LOST_FRAMES           = OTX_ATTR_END +  6,  // @emem (OTX_ATTRIB_UINT32) Number of lost frames (due to Fifo-full condition)

  OTX_ATTR_HDLC_RECEIVER_END                        = OTX_ATTR_END +  7
};

typedef enum OtxHdlcReceiverAttributesE OtxHdlcReceiverAttributesE;


/*---------------------------------------------------------------------------*\
  @enum
    HDLC Receiver Event Codes
\*---------------------------------------------------------------------------*/
enum OtxHdlcReceiverEventCodesE {
  OTX_HDLC_RECEIVER_EC_COMPLETE_FRAME      = 0x00, // @emem HDLC Frame Received
  OTX_HDLC_RECEIVER_EC_START_FRAME         = 0x01, // @emem HDLC Receive Frame Start
  OTX_HDLC_RECEIVER_EC_PROTOCOL_ERROR      = 0x02, // @emem HDLC Protocol Error
  OTX_HDLC_RECEIVER_EC_BUFFER_OVERRUN      = 0x03, // @emem HDLC Buffer Overrun. The host is too slow retrieving the data from the Hdlc Receiver

  OTX_HDLC_RECEIVER_END_OF_EVENT_CODES
};

typedef enum OtxHdlcReceiverEventCodesE OtxHdlcReceiverEventCodesE;


/*---------------------------------------------------------------------------*\
  @enum
    HDLC Receiver Error Result Codes
\*---------------------------------------------------------------------------*/
enum OtxHdlcReceiverResultCodesE {
  OTX_HDLC_RECEIVER_RES_END = OTX_RES_END + 0
};

typedef enum OtxHdlcReceiverResultCodesE OtxHdlcReceiverResultCodesE;



/*---------------------------------------------------------------------------*\
  @enum 
    HDLC Sender Attributes
\*---------------------------------------------------------------------------*/
enum OtxHdlcSenderAttributesE {
  OTX_ATTR_HDLC_SENDER_GENERATE_FRAME_SENT_EVENT    = OTX_ATTR_END + 0,  // @emem (OTX_ATTRIB_BOOL) Read/Write. If OTX_TRUE the HDLC Sender will generate a OTX_HDLC_SENDER_EC_FRAME_SENT event after each transmitted frame

  OTX_ATTR_HDLC_SENDER_MAX_QUEUED_PACKETS           = OTX_ATTR_END + 1,  // @emem (OTX_ATTRIB_UINT32) Read/Write for DSP hosted devices. Read-only for T1E1 hosted devices.
                                                                         // Max number of packets in the Transmit Data queue (in the host device). 
                                                                         // By increasing the value of this attribute the Raw Data Sender device
                                                                         // will be able to store more packets in the DSP code thus becoming less
                                                                         // suseptible to underrun errors due to a long host interrupt latency.
                                                                         // This value must be set before <f OtxHdlcSenderInit> is called.
                                                                    
  OTX_ATTR_HDLC_SENDER_QUEUE_NOTIFY_LEVEL           = OTX_ATTR_END + 2,  // @emem (OTX_ATTRIB_UINT32) Read/Write for DSP hosted devices. Read-only for T1E1 hosted devices.
                                                                         // The host is notified when the Transmit Data queue (in the host device)
                                                                         // is depleted to this many packets.
                                                                         // This value must be set before <f OtxHdlcSenderInit> is called.
                                                                    
  OTX_ATTR_HDLC_SENDER_QUEUE_MAX_PACKET_SIZE        = OTX_ATTR_END + 3,  // @emem (OTX_ATTRIB_UINT32) Read-only.
                                                                         // Maximum number of octets in each transmit packet that the host will
                                                                         // deliver to the the host device. This value is set by the host device, and
                                                                         // its value is a function of the pipe capacity (higher pipe capacity means higher
                                                                         // bigger packet size).
                                                                    

  OTX_ATTR_HDLC_SENDER_END                          = OTX_ATTR_END + 1
};

typedef enum OtxHdlcSenderAttributesE OtxHdlcSenderAttributesE;


/*---------------------------------------------------------------------------*\
  @enum
    HDLC Sender Event Codes
\*---------------------------------------------------------------------------*/
enum OtxHdlcSenderEventCodesE {
  OTX_HDLC_SENDER_EC_FRAME_SENT      = 0x00, // @emem HDLC Frame completely sent
                                             // (<e OtxEventDataS.m_nRequestId> is the <p nTaskRef> as passed in <f OtxHdlcSendData> call). 
                                             // If generation of this event is desired, it must be activated by setting the
                                             // <e OtxHdlcSenderAttributesE.OTX_ATTR_HDLC_SENDER_GENERATE_DATA_EVENTS> to OTX_TRUE
                                             // before <f OtxHdlcSenderInit> is called.

  OTX_HDLC_SENDER_EC_BUF_UNDERRUN    = 0x01, // @emem Buffer Underrun. Host is to slow supplying HDLC data to the Hdlc Sender

  OTX_HDLC_SENDER_EC_FRAME_LOST      = 0x02, // @emem Internal device error. Frame is lost. This could occur if
                                             // the host (DSP code) of the device runs out of dynamic memory for the packet
                                             // to be transmitted.

  OTX_HDLC_SENDER_EC_DEBUG_INFO      = 0x03, // @emem Internal use. For driver debugging.

  OTX_HDLC_SENDER_END_OF_EVENT_CODES
};

typedef enum OtxHdlcSenderEventCodesE OtxHdlcSenderEventCodesE;

/*---------------------------------------------------------------------------*\
  @enum
    HDLC Sender Error Result Codes
\*---------------------------------------------------------------------------*/
enum OtxHdlcSenderResultCodesE {
  OTX_HDLC_SENDER_RES_TX_BUSY                  = OTX_RES_END + 0,  // @emem Transmitter is busy sending a frame.
  OTX_HDLC_SENDER_RES_FRAME_TOO_LONG           = OTX_RES_END + 1,  // @emem Frame too long.

  OTX_HDLC_SENDER_RES_END                      = OTX_RES_END + 2
};

typedef enum OtxHdlcSenderResultCodesE OtxHdlcSenderResultCodesE;



/*---------------------------------------------------------------------------*\
  @enum
    Frame End Code for HDLC frames. Specifies whether the frame is a good or
    faulty frame.
    If the frame is a faulty, then this bit mask provides further indications of the cause 
    of the fault. 
    The frame End Code of frame is retrieved by a call to <f OtxHdlcReadData>. 
    Please note that the Frame End Code is a bit mask value. The Frame End code can 
    have multiple values, e.g. both <e OtxHdlcFrameEndCodeE.OTX_HDLC_FEC_INVALID> and 
    <e OtxHdlcFrameEndCodeE.OTX_HDLC_FEC_ABORTED> can be set.
    
  @xref
    <f OtxHdlcReadData>

\*---------------------------------------------------------------------------*/
enum OtxHdlcFrameEndCodeE {
  OTX_HDLC_FEC_OK         = 0x00,         // @emem The frame is a good HDLC frame. No faults
  OTX_HDLC_FEC_INVALID    = 0x01,         // @emem Invalid frame; Either not a multiple of 8 bits, or a short frame
  OTX_HDLC_FEC_OVERFLOW   = 0x02,         // @emem Receive Data Overflow. The host is not servicing the receive FIFO fast enough.
  OTX_HDLC_FEC_BAD_CRC    = 0x04,         // @emem The frame failed the CRC check
  OTX_HDLC_FEC_ABORTED    = 0x10,         // @emem The received frame was aborted
  OTX_HDLC_FEC_TRUNCATED  = 0x20,         // @emem The received frame was too long for the buffer in the driver
}; 

typedef enum OtxHdlcFrameEndCodeE OtxHdlcFrameEndCodeE;



// @enum Character(s) to be send between HDLC frames
enum OtxHdlcFrameFillTypeE {
  OTX_THOR_FFT_NONE     = 0,  // @emem No frame fill between frames
  OTX_HDLC_FFT_FLAGS    = 1,  // @emem Send HDLC flags (0x7E) between the frames.
  OTX_HDLC_FFT_ALL_ONES = 2   // @emem Send all ones (0xFF) between frames.
};

typedef enum OtxHdlcFrameFillTypeE OtxHdlcFrameFillTypeE;


// @enum Cyclic Redundancy Check (CRC) mode
enum OtxHdlcCrcModeE {
  OTX_HDLC_CRC16    = 0,  // @emem 16-bit Cyclic Redundancy Check
  OTX_HDLC_CRC32    = 1   // @emem 32-bit Cyclic Redundancy Check
};

typedef enum OtxHdlcCrcModeE OtxHdlcCrcModeE;


// @enum HDLC Transmission Rates for V.110 and X.30 modes
enum OtxHdlcTxRateE {
  OTX_HDLC_TR_NA      = 0x04, // @emem Transmission Rate Not Applicaple in this Pipe Mode
  OTX_HDLC_TR_600BPS  = 0x00, // @emem Transmission Rate 600 bit/s
  OTX_HDLC_TR_1200BPS = 0x01, // @emem Transmission Rate 1200 bit/s
  OTX_HDLC_TR_2400BPS = 0x02, // @emem Transmission Rate 2400 bit/s
  OTX_HDLC_TR_4800BPS = 0x03, // @emem Transmission Rate 4800 bit/s
};

typedef enum OtxHdlcTxRateE OtxHdlcTxRateE;


// @struct Fill-In Signalling Unit (FISU), used in the <t OTX_HDLC_MODE_SS7> mode
struct OtxHdlcSS7FisuS {
  unsigned int bsn : 7;       // @field Backward Sequence Number
  unsigned int bib : 1;       // @field Backward Indicator Bit
  unsigned int fsn : 7;       // @field Forward Sequence Number
  unsigned int fib : 1;       // @field Forward Indicator Bit
  unsigned int li  : 6;       // @field Length Indication (0)
  unsigned int reserved : 2;  // @field Reserved
};

typedef struct OtxHdlcSS7FisuS OtxHdlcSS7FisuS;

// @struct Short Link Status Signalling Unit (Short LSSU), used in the <t OTX_HDLC_MODE_SS7> mode
struct OtxHdlcSS7ShortLssuS {
  unsigned int bsn : 7;       // @field Backward Sequence Number
  unsigned int bib : 1;       // @field Backward Indicator Bit
  unsigned int fsn : 7;       // @field Forward Sequence Number
  unsigned int fib : 1;       // @field Forward Indicator Bit
  unsigned int li  : 6;       // @field Length Indication (1)
  unsigned int reserved : 2;  // @field Reserved
  unsigned int sf  : 8;       // @field Status Field
};

typedef struct OtxHdlcSS7ShortLssuS OtxHdlcSS7ShortLssuS;

// @struct Long Link Status Signalling Unit (Long LSSU), used in the <t OTX_HDLC_MODE_SS7> mode
struct OtxHdlcSS7LongLssuS {
  unsigned int bsn : 7;       // @field Backward Sequence Number
  unsigned int bib : 1;       // @field Backward Indicator Bit
  unsigned int fsn : 7;       // @field Forward Sequence Number
  unsigned int fib : 1;       // @field Forward Indicator Bit
  unsigned int li  : 6;       // @field Length Indication (2)
  unsigned int reserved : 2;  // @field Reserved
  unsigned int sf  : 16;      // @field Status Field
};

typedef struct OtxHdlcSS7LongLssuS OtxHdlcSS7LongLssuS;

// @struct HDLC info data structs preceeding HDLC data in buffer returned by <f HdlcReadDataEx>.
// Each frame in the buffer is preceeded with a struct of this type. 
struct OtxHdlcFrameInfoS 
{
  OTX_UINT32    cbOctets;              // @field Number of octets in the frame (not including this struct)
  OTX_UINT32    nFrameSeq;             // @field Sequence number of the received HDLC frame (matches the number in the event queue)
  OSA_ENUM_DECL(OtxHdlcFrameEndCodeE)  nFrameEndCode;         // @field Frame Status Code
  OTX_DATETIME  nTimeStamp;            // @field Timestamp of the last received octet of the frame
  OTX_UINT32    nOffsetNext;           // @field Offset in bytes to the next OtxHdlcFrameInfoS realtive the start of this one or, 0 if this one is last
	OTX_UINT8     sqData[OTX_VARI_SIZE_FIELD];
};


typedef struct OtxHdlcFrameInfoS OtxHdlcFrameInfoS;

#ifdef __linux__
#pragma pack()
#else
#pragma pack(pop)
#endif

#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998 - 2001


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/

#endif // OTX_HDLC_DEF_H
