/* $Workfile: OtxRawD.h $   $Revision: 6 $    $Modtime: 9/07/01 5:40p $  */
#ifndef OTX_RAWDATA_DEF_H
#define OTX_RAWDATA_DEF_H
/*******************************************************************************

  @doc PublicHwApi OtxRawD

  Copyright (C) 1999 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic Hardware Driver API

  @module                                        
  OtxRawD | Constants, Macros, and Types for the Raw Data Functions

  @contents2
  OtxRawD.h |

  @index enum type struct | OtxRawD

  @contents2
  OTX Raw Data Device Macros, Constants, and Types |

  @head3 OtxRawDataXXX Macro, Constant, and Type Reference |

  @index enum type struct | OtxRawD

*******************************************************************************/

#ifdef __cplusplus
  extern "C" {
#endif

#if 0
#define OTX_RAWDATA_MAX_TX_FRAME_LEN    8192
#define OTX_RAWDATA_MAX_RX_FRAME_LEN    8192
#else // Temporary reduced. Will be brought back to 8192 in future releases
#define OTX_RAWDATA_MAX_TX_FRAME_LEN    2048
//#define OTX_RAWDATA_MAX_RX_FRAME_LEN    2048
#endif

/*---------------------------------------------------------------------------*\
  @enum 
    RAWDATA Receiver Attributes
\*---------------------------------------------------------------------------*/
enum OtxRawDataReceiverAttributesE {
  OTX_ATTR_RAWDATA_RECEIVER_FIFO_SIZE = OTX_ATTR_END + 0,            // @emem (OTX_ATTRIB_UINT32) Read/Write.
                                                                    // Fifo buffer size factor for received data in the driver.
                                                                    // The fifo is allocated when the logical device is connected.
                                                                    // Total fifo size in bytes is this attribute * (PipeCapacity/64).
  OTX_ATTR_RAWDATA_RECEIVER_GENERATE_DATA_EVENTS = OTX_ATTR_END + 1,  // @emem (OTX_ATTRIB_BOOL) Read/Write.
                                                                    // If set to OTX_TRUE (default) the device will generate
                                                                    // a <e OtxRawDataReceiverEventCodesE.OTX_RAWDATA_RECEIVER_EC_DATA_AVAILABLE> 
                                                                    // event when the device has any data to be retrieved from the driver buffer.
                                                                    // By setting this attribute to OTX_FALSE, such events
                                                                    // are not generated, thus reducing event handling processing overhead.

  OTX_ATTR_RAWDATA_RECEIVER_FIFO_SIZE_FRAMES           = OTX_ATTR_END +  2,  // @emem (OTX_ATTRIB_UINT32) Read only. 
                                                                             // Size of allocated buffer for the RAWDATA Receiver Fifo (in number of frames)
  OTX_ATTR_RAWDATA_RECEIVER_FIFO_SIZE_OCTETS           = OTX_ATTR_END +  3,  // @emem (OTX_ATTRIB_UINT32)  Read only.
                                                                             // Size of allocated buffer for the RAWDATA Receiver Fifo (in number of bytes)
  OTX_ATTR_RAWDATA_RECEIVER_FIFO_PEAK_USAGE_OCTETS     = OTX_ATTR_END +  4,  // @emem (OTX_ATTRIB_UINT32)  Read only.
                                                                             // Peak usage (in octets) of the RAWDATA Receiver Fifo
  OTX_ATTR_RAWDATA_RECEIVER_FIFO_PEAK_USAGE_FRAMES     = OTX_ATTR_END +  5,  // @emem (OTX_ATTRIB_UINT32)  Read only.
                                                                             // Peak usage (in frames) of the RAWDATA Receiver Fifo
  OTX_ATTR_RAWDATA_RECEIVER_FIFO_CURRENT_USAGE_OCTETS  = OTX_ATTR_END +  6,  // @emem (OTX_ATTRIB_UINT32)  Read only.
                                                                             // Current usage (in octets) of the RAWDATA Receiver Fifo
  OTX_ATTR_RAWDATA_RECEIVER_FIFO_CURRENT_USAGE_FRAMES  = OTX_ATTR_END +  7,  // @emem (OTX_ATTRIB_UINT32)  Read only.
                                                                             // Current usage (in frames) of the RAWDATA Receiver Fifo
  OTX_ATTR_RAWDATA_RECEIVER_FIFO_LOST_FRAMES           = OTX_ATTR_END +  8,  // @emem (OTX_ATTRIB_UINT32)  Read only.
                                                                             // Number of lost frames (due to Fifo-full condition)

  OTX_ATTR_RAWDATA_RECEIVER_END                 = OTX_ATTR_END + 2
};
typedef enum OtxRawDataReceiverAttributesE OtxRawDataReceiverAttributesE;


/*---------------------------------------------------------------------------*\
  @enum
    RAWDATA Receiver Event Codes
\*---------------------------------------------------------------------------*/
enum OtxRawDataReceiverEventCodesE {
  OTX_RAWDATA_RECEIVER_EC_DATA_AVAILABLE      = 0x00, // @emem Raw Data is available to 
                                                      // be retrieved by calling <f OtxRawDataReadData>
                                                      // Generation of this event can be blocked by setting the
                                                      // <e OtxRawDataReceiverAttributesE.OTX_ATTR_RAWDATA_RECEIVER_GENERATE_DATA_EVENTS>
                                                      // attribute to OTX_FALSE.

  OTX_RAWDATA_RECEIVER_EC_BUFFER_OVERRUN      = 0x01, // @emem Raw Data Buffer Overrun. 
                                                      // The host is too slow retrieving 
                                                      // the data from the Raw Data Receiver

  OTX_RAWDATA_RECEIVER_EC_DEBUG_INFO          = 0x02, // @emem Internal use. For driver debugging.

  OTX_RAWDATA_RECEIVER_EC_EXTRACTING_FLAGS    = 0x03, // @emem Flag encountered. Filtering of flags started.

  OTX_RAWDATA_RECEIVER_EC_BUFFERING_DATA      = 0x04, // @emem Non-flag octet encountered. Ended filtering of flags. Starting buffering data.


  OTX_RAWDATA_RECEIVER_END_OF_EVENT_CODES
};

typedef enum OtxRawDataReceiverEventCodesE OtxRawDataReceiverEventCodesE;


/*---------------------------------------------------------------------------*\
  @enum
    RAWDATA Receiver Error Result Codes
\*---------------------------------------------------------------------------*/
enum OtxRawDataReceiverResultCodesE {

  OTX_RAWDATA_RECEIVER_RES_END = OTX_RES_END + 0
};

typedef enum OtxRawDataReceiverResultCodesE OtxRawDataReceiverResultCodesE;


/*---------------------------------------------------------------------------*\
  @enum 
    RAWDATA Sender Attributes
\*---------------------------------------------------------------------------*/
enum OtxRawDataSenderAttributesE {
  OTX_ATTR_RAWDATA_SENDER_GENERATE_DATA_EVENTS  = OTX_ATTR_END + 0,  // @emem (OTX_ATTRIB_BOOL) Read/Write.
                                                                     // If set to OTX_TRUE (default) the device will generate
                                                                     // a <e OtxRawDataSenderEventCodesE.OTX_RAWDATA_SENDER_EC_BUF_DEPLETED> 
                                                                     // event when the device is ready
                                                                     // to accept more data. By setting this attribute to OTX_FALSE, such events
                                                                     // are not generated, thus reducing event handling processing overhead.
                                                                    
  OTX_ATTR_RAWDATA_SENDER_MAX_QUEUED_PACKETS    = OTX_ATTR_END + 1,  // @emem (OTX_ATTRIB_UINT32) Read/Write.
                                                                     // Max number of packets in the Transmit Data queue (in the DSP device). 
                                                                     // By increasing the value of this attribute the Raw Data Sender device
                                                                     // will be able to store more packets in the DSP code thus becoming less
                                                                     // suseptible to underrun errors due to a long host interrupt latency.
                                                                     // This value must be set before <f OtxRawDataSenderInit> is called.
                                                                    
  OTX_ATTR_RAWDATA_SENDER_QUEUE_NOTIFY_LEVEL    = OTX_ATTR_END + 2,  // @emem (OTX_ATTRIB_UINT32) Read/Write.
                                                                     // The host is notified when the Transmit Data queue (in the DSP device)
                                                                     // is depleted to this many packets.
                                                                     // This value must be set before <f OtxRawDataSenderInit> is called.
                                                                    
  OTX_ATTR_RAWDATA_SENDER_QUEUE_MAX_PACKET_SIZE = OTX_ATTR_END + 3,  // @emem (OTX_ATTRIB_UINT32) Read-only.
                                                                     // Maximum number of octets in each transmit packet that the host will
                                                                     // deliver to the the DSP device. This value is set by the DSP code, and
                                                                     // its value is a function of the pipe capacity (higher pipe capacity means higher
                                                                     // bigger packet size).
                                                                    
  OTX_ATTR_RAWDATA_SENDER_END                   = OTX_ATTR_END + 4
};
typedef enum OtxRawDataSenderAttributesE OtxRawDataSenderAttributesE;

/*---------------------------------------------------------------------------*\
  @enum
    Raw Data Sender Event Codes
\*---------------------------------------------------------------------------*/
enum OtxRawDataSenderEventCodesE {
  OTX_RAWDATA_SENDER_EC_COMPLETED      = 0x00, // @emem Raw Data completely sent. 
                                               // The device is now sending flags.
                                               // This event will also occur if the host is too slow to provide data
                                               // to the Raw Data Sender device (i.e. when the device is starving for data).
  OTX_RAWDATA_SENDER_EC_BUF_DEPLETED   = 0x01, // @emem Raw Data Sender has completed sending a buffer 
                                               // (the buffer with <e OtxEventDataS.m_nRequestId> as passed <p nTaskRef> in <f OtxRawDataSendData>). 
                                               // The device is now accepting more data.
                                               // Generation of this event can be blocked by setting the
                                               // <e OtxRawDataSenderAttributesE.OTX_ATTR_RAWDATA_SENDER_GENERATE_DATA_EVENTS>
                                               // attribute to OTX_FALSE before <f OtxRawDataSenderInit> is called.
  OTX_RAWDATA_SENDER_EC_DATA_LOST      = 0x02, // @emem Internal device error. Frame is lost. This could occur if
                                               // the host (DSP code) of the device runs out of dynamic memory for the packet
                                               // to be transmitted.
  OTX_RAWDATA_SENDER_EC_DEBUG_INFO     = 0x03, // @emem Internal use. For driver debugging.


  OTX_RAWDATA_SENDER_END_OF_EVENT_CODES
};

typedef enum OtxRawDataSenderEventCodesE OtxRawDataSenderEventCodesE;

/*---------------------------------------------------------------------------*\
  @enum
    Raw Data Sender Error Result Codes
\*---------------------------------------------------------------------------*/
enum OtxRawDataSenderResultCodesE {
  OTX_RAWDATA_SENDER_RES_TX_BUSY                  =  OTX_RES_END +  0,  // @emem Transmitter queue is full.
  OTX_RAWDATA_SENDER_RES_FRAME_TOO_LONG           =  OTX_RES_END +  1,  // @emem Data buffer too long.

  OTX_RAWDATA_SENDER_RES_END                      =  OTX_RES_END +  2
};

typedef enum OtxRawDataSenderResultCodesE OtxRawDataSenderResultCodesE;



#ifdef __cplusplus
  } // extern "C"
#endif


/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1999


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/

#endif // OTX_RAWDATA_DEF_H
